@@ -17,7 +17,7 @@ import sys
 import tempfile
 import warnings
 import weakref
-from collections.abc import Iterable
+from collections.abc import Callable, Iterable
 from datetime import datetime, timedelta, timezone
 from functools import lru_cache
 from hashlib import sha384
@@ -28,22 +28,20 @@ from threading import Thread
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
     ClassVar,
     Literal,
-    Optional,
-    Union,
+    TypeAlias,
     cast,
 )
 from urllib.error import HTTPError, URLError
 from urllib.request import Request, urlopen
 from zipfile import BadZipFile, ZipFile
 
-from hypothesis._settings import note_deprecation
 from hypothesis.configuration import storage_directory
 from hypothesis.errors import HypothesisException, HypothesisWarning
 from hypothesis.internal.conjecture.choice import ChoiceT
 from hypothesis.utils.conventions import UniqueIdentifier, not_set
+from hypothesis.utils.deprecation import note_deprecation
 
 __all__ = [
     "DirectoryBasedExampleDatabase",
@@ -55,17 +53,15 @@ __all__ = [
 ]
 
 if TYPE_CHECKING:
-    from typing import TypeAlias
-
     from watchdog.observers.api import BaseObserver
 
-StrPathT: "TypeAlias" = Union[str, PathLike[str]]
-SaveDataT: "TypeAlias" = tuple[bytes, bytes]  # key, value
-DeleteDataT: "TypeAlias" = tuple[bytes, Optional[bytes]]  # key, value
-ListenerEventT: "TypeAlias" = Union[
-    tuple[Literal["save"], SaveDataT], tuple[Literal["delete"], DeleteDataT]
-]
-ListenerT: "TypeAlias" = Callable[[ListenerEventT], Any]
+StrPathT: TypeAlias = str | PathLike[str]
+SaveDataT: TypeAlias = tuple[bytes, bytes]  # key, value
+DeleteDataT: TypeAlias = tuple[bytes, bytes | None]  # key, value
+ListenerEventT: TypeAlias = (
+    tuple[Literal["save"], SaveDataT] | tuple[Literal["delete"], DeleteDataT]
+)
+ListenerT: TypeAlias = Callable[[ListenerEventT], Any]
 
 
 def _usable_dir(path: StrPathT) -> bool:
@@ -80,12 +76,14 @@ def _usable_dir(path: StrPathT) -> bool:
             # Loop terminates because the root dir ('/' on unix) always exists.
             path = path.parent
         return path.is_dir() and os.access(path, os.R_OK | os.W_OK | os.X_OK)
-    except PermissionError:
+    except PermissionError:  # pragma: no cover
+        # path.exists() returns False on 3.14+ instead of raising. See
+        # https://docs.python.org/3.14/library/pathlib.html#querying-file-type-and-status
         return False
 
 
 def _db_for_path(
-    path: Optional[Union[StrPathT, UniqueIdentifier, Literal[":memory:"]]] = None,
+    path: StrPathT | UniqueIdentifier | Literal[":memory:"] | None = None,
 ) -> "ExampleDatabase":
     if path is not_set:
         if os.getenv("HYPOTHESIS_DATABASE_FILE") is not None:  # pragma: no cover
@@ -136,11 +134,36 @@ class _EDMeta(abc.ABCMeta):
 # This code only runs if Sphinx has already been imported; and it would live in our
 # docs/conf.py except that we would also like it to work for anyone documenting
 # downstream ExampleDatabase subclasses too.
-if "sphinx" in sys.modules:
+#
+# We avoid type-checking this block due to this combination facts:
+# * our check-types-api CI job runs under 3.14
+# * tools.txt therefore pins to a newer version of sphinx which uses 3.12+ `type`
+#   syntax
+# * in test_mypy.py, mypy sees this block, sees sphinx is installed, tries parsing
+#   sphinx code, and errors
+#
+# Putting `and not TYPE_CHECKING` here is just a convenience for our testing setup
+# (because we don't split mypy tests by running CI version, eg), not for runtime
+#  behavior.
+if "sphinx" in sys.modules and not TYPE_CHECKING:  # pragma: no cover
     try:
-        from sphinx.ext.autodoc import _METACLASS_CALL_BLACKLIST
+        import sphinx.ext.autodoc
 
-        _METACLASS_CALL_BLACKLIST.append("hypothesis.database._EDMeta.__call__")
+        signature = "hypothesis.database._EDMeta.__call__"
+
+        # _METACLASS_CALL_BLACKLIST moved in newer sphinx versions
+        try:
+            import sphinx.ext.autodoc._dynamic._signatures as _module
+        except ImportError:
+            _module = sphinx.ext.autodoc
+
+        # _METACLASS_CALL_BLACKLIST is a frozenset in later sphinx versions
+        if isinstance(_module._METACLASS_CALL_BLACKLIST, frozenset):
+            _module._METACLASS_CALL_BLACKLIST = _module._METACLASS_CALL_BLACKLIST | {
+                signature
+            }
+        else:
+            _module._METACLASS_CALL_BLACKLIST.append(signature)
     except Exception:
         pass
 
@@ -447,11 +470,16 @@ class DirectoryBasedExampleDatabase(ExampleDatabase):
         kp = self._key_path(key)
         if not kp.is_dir():
             return
-        for path in os.listdir(kp):
-            try:
-                yield (kp / path).read_bytes()
-            except OSError:
-                pass
+
+        try:
+            for path in os.listdir(kp):
+                try:
+                    yield (kp / path).read_bytes()
+                except OSError:
+                    pass
+        except OSError:  # pragma: no cover
+            # the `kp` directory might have been deleted in the meantime
+            pass
 
     def save(self, key: bytes, value: bytes) -> None:
         key_path = self._key_path(key)
@@ -552,10 +580,10 @@ class DirectoryBasedExampleDatabase(ExampleDatabase):
         _metakeys_hash = self._metakeys_hash
         _broadcast_change = self._broadcast_change
 
-        class Handler(FileSystemEventHandler):
-            def on_created(
-                _self, event: Union[FileCreatedEvent, DirCreatedEvent]
-            ) -> None:
+        class Handler(
+            FileSystemEventHandler
+        ):  # pragma: no cover # skipped in test_database.py for now
+            def on_created(_self, event: FileCreatedEvent | DirCreatedEvent) -> None:
                 # we only registered for the file creation event
                 assert not isinstance(event, DirCreatedEvent)
                 # watchdog events are only bytes if we passed a byte path to
@@ -567,7 +595,13 @@ class DirectoryBasedExampleDatabase(ExampleDatabase):
                 key_hash = value_path.parent.name
 
                 if key_hash == _metakeys_hash:
-                    hash_to_key[value_path.name] = value_path.read_bytes()
+                    try:
+                        hash_to_key[value_path.name] = value_path.read_bytes()
+                    except OSError:  # pragma: no cover
+                        # this might occur if all the values in a key have been
+                        # deleted and DirectoryBasedExampleDatabase removes its
+                        # metakeys entry (which is `value_path` here`).
+                        pass
                     return
 
                 key = hash_to_key.get(key_hash)
@@ -583,9 +617,7 @@ class DirectoryBasedExampleDatabase(ExampleDatabase):
 
                 _broadcast_change(("save", (key, value)))
 
-            def on_deleted(
-                self, event: Union[FileDeletedEvent, DirDeletedEvent]
-            ) -> None:
+            def on_deleted(self, event: FileDeletedEvent | DirDeletedEvent) -> None:
                 assert not isinstance(event, DirDeletedEvent)
                 assert isinstance(event.src_path, str)
 
@@ -596,7 +628,7 @@ class DirectoryBasedExampleDatabase(ExampleDatabase):
 
                 _broadcast_change(("delete", (key, None)))
 
-            def on_moved(self, event: Union[FileMovedEvent, DirMovedEvent]) -> None:
+            def on_moved(self, event: FileMovedEvent | DirMovedEvent) -> None:
                 assert not isinstance(event, DirMovedEvent)
                 assert isinstance(event.src_path, str)
                 assert isinstance(event.dest_path, str)
@@ -824,7 +856,7 @@ class GitHubArtifactDatabase(ExampleDatabase):
         repo: str,
         artifact_name: str = "hypothesis-example-db",
         cache_timeout: timedelta = timedelta(days=1),
-        path: Optional[StrPathT] = None,
+        path: StrPathT | None = None,
     ):
         super().__init__()
         self.owner = owner
@@ -834,7 +866,7 @@ class GitHubArtifactDatabase(ExampleDatabase):
 
         # Get the GitHub token from the environment
         # It's unnecessary to use a token if the repo is public
-        self.token: Optional[str] = getenv("GITHUB_TOKEN")
+        self.token: str | None = getenv("GITHUB_TOKEN")
 
         if path is None:
             self.path: Path = Path(
@@ -849,9 +881,9 @@ class GitHubArtifactDatabase(ExampleDatabase):
 
         # This is the path to the artifact in usage
         # .hypothesis/github-artifacts/<artifact-name>/<modified_isoformat>.zip
-        self._artifact: Optional[Path] = None
+        self._artifact: Path | None = None
         # This caches the artifact structure
-        self._access_cache: Optional[dict[PurePath, set[PurePath]]] = None
+        self._access_cache: dict[PurePath, set[PurePath]] | None = None
 
         # Message to display if user doesn't wrap around ReadOnlyDatabase
         self._read_only_message = (
@@ -972,7 +1004,7 @@ class GitHubArtifactDatabase(ExampleDatabase):
 
         self._prepare_for_io()
 
-    def _get_bytes(self, url: str) -> Optional[bytes]:  # pragma: no cover
+    def _get_bytes(self, url: str) -> bytes | None:  # pragma: no cover
         request = Request(
             url,
             headers={
@@ -982,7 +1014,7 @@ class GitHubArtifactDatabase(ExampleDatabase):
             },
         )
         warning_message = None
-        response_bytes: Optional[bytes] = None
+        response_bytes: bytes | None = None
         try:
             with urlopen(request) as response:
                 response_bytes = response.read()
@@ -995,9 +1027,11 @@ class GitHubArtifactDatabase(ExampleDatabase):
             else:
                 warning_message = (
                     "Could not get the latest artifact from GitHub. "
-                    "This could be because because the repository "
+                    "This could be because the repository "
                     "or artifact does not exist. "
                 )
+            # see https://github.com/python/cpython/issues/128734
+            e.close()
         except URLError:
             warning_message = "Could not connect to GitHub to get the latest artifact. "
         except TimeoutError:
@@ -1012,7 +1046,7 @@ class GitHubArtifactDatabase(ExampleDatabase):
 
         return response_bytes
 
-    def _fetch_artifact(self) -> Optional[Path]:  # pragma: no cover
+    def _fetch_artifact(self) -> Path | None:  # pragma: no cover
         # Get the list of artifacts from GitHub
         url = f"https://api.github.com/repos/{self.owner}/{self.repo}/actions/artifacts"
         response_bytes = self._get_bytes(url)
@@ -1071,7 +1105,7 @@ class GitHubArtifactDatabase(ExampleDatabase):
         kp = self._key_path(key)
 
         with ZipFile(self._artifact) as zf:
-            # Get the all files in the the kp from the cache
+            # Get all the files in the kp from the cache
             filenames = self._access_cache.get(kp, ())
             for filename in filenames:
                 with zf.open(filename.as_posix()) as f:
@@ -1099,7 +1133,7 @@ class BackgroundWriteDatabase(ExampleDatabase):
         super().__init__()
         self._db = db
         self._queue: Queue[tuple[str, tuple[bytes, ...]]] = Queue()
-        self._thread: Optional[Thread] = None
+        self._thread: Thread | None = None
 
     def _ensure_thread(self):
         if self._thread is None:
@@ -1121,7 +1155,7 @@ class BackgroundWriteDatabase(ExampleDatabase):
             getattr(self._db, method)(*args)
             self._queue.task_done()
 
-    def _join(self, timeout: Optional[float] = None) -> None:
+    def _join(self, timeout: float | None = None) -> None:
         # copy of Queue.join with a timeout. https://bugs.python.org/issue9634
         with self._queue.all_tasks_done:
             while self._queue.unfinished_tasks:
@@ -1242,7 +1276,7 @@ def _choices_from_bytes(buffer: bytes, /) -> tuple[ChoiceT, ...]:
             parts.append(bool(size))
             continue
         if size == 0b11111:
-            (offset, size) = _unpack_uleb128(buffer[idx:])
+            offset, size = _unpack_uleb128(buffer[idx:])
             idx += offset
         chunk = buffer[idx : idx + size]
         idx += size
@@ -1260,7 +1294,7 @@ def _choices_from_bytes(buffer: bytes, /) -> tuple[ChoiceT, ...]:
     return tuple(parts)
 
 
-def choices_from_bytes(buffer: bytes, /) -> Optional[tuple[ChoiceT, ...]]:
+def choices_from_bytes(buffer: bytes, /) -> tuple[ChoiceT, ...] | None:
     """
     Deserialize a bytestring to a tuple of choices. Inverts choices_to_bytes.
 
