@@ -27,7 +27,6 @@ from typing import (
 import numpy as np
 
 from hypothesis import strategies as st
-from hypothesis._settings import note_deprecation
 from hypothesis.errors import HypothesisException, InvalidArgument
 from hypothesis.extra._array_helpers import (
     _BIE,
@@ -60,6 +59,7 @@ from hypothesis.strategies._internal.strategies import (
     check_strategy,
 )
 from hypothesis.strategies._internal.utils import defines_strategy
+from hypothesis.utils.deprecation import note_deprecation
 
 
 def _try_import(mod_name: str, attr_name: str) -> Any:
@@ -1166,7 +1166,7 @@ def basic_indices(
     * ``min_dims`` is the minimum dimensionality of the resulting array from use
       of the generated index. When ``min_dims == 0``, scalars and zero-dimensional
       arrays are both allowed.
-    * ``max_dims`` is the the maximum dimensionality of the resulting array,
+    * ``max_dims`` is the maximum dimensionality of the resulting array,
       defaulting to ``len(shape) if not allow_newaxis else
       max(len(shape), min_dims) + 2``.
     * ``allow_newaxis`` specifies whether ``None`` is allowed in the index.
@@ -1433,7 +1433,9 @@ def _from_type(thing: type[Ex]) -> st.SearchStrategy[Ex] | None:
             st.recursive(st.tuples(base_strat, base_strat), st.tuples),
         )
 
-    if origin in [np.ndarray, _SupportsArray]:
+    # note: get_origin(np.typing.NDArray[np.int64]) is np.ndarray in numpy < 2.5.0,
+    # but is np.typing.NDArray in numpy >= 2.5.0. Support both here.
+    if origin in [np.typing.NDArray, np.ndarray, _SupportsArray]:
         dtype = _dtype_from_args(args)
         return arrays(dtype, array_shapes(max_dims=2))  # type: ignore[return-value]
 
